/*===========================================================

	the firmware of USB keyboard adapter for PC-8801 mkII
	(c) 2010 Koichi NISHIDA

===========================================================*/

/*
hardware information:

use ATTINY2313 AVR.
connect 27MHz (overclock...) crystal to the AVR.
supply 3.3V power.
fuse setting : LOW 11011110

connection:
	PD0:	connect to USB D+ through 33 ohm register, and pull down with 15k ohm register.
	PD1:	connect to USB D- through 33 ohm register, and pull down with 15k ohm register .
 	PD2-D5:	scan input from 88 mkII, and pull up with 10K ohm register.
	PD6:	pull down.
	PB0-7:	output to 88 mkII.
*/

/*
This is the firmware of USB keyboard adapter for PC-8801 mkII by Nishida Radio.

Copyright (C) 2010 Koichi NISHIDA
email to Koichi NISHIDA: tulip-house@msf.biglobe.ne.jp

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

	.include "tn2313def.inc"

	; use ATTINY2313 at 27MHz: 18 clock/bit for USB low speed (1.5 M BPS)

	.equ	TIMER_INTV	= 105	; 1m sec (sending EOP interval)
	.equ	INTR_INTV	= 30	; 30 msec (interrupt transfer interval)
	.equ	SIMUL_PUSH 	= 3		; simulatenous key pushes

	; work area
	.equ	TCOUNT		= 0x60	; for conting INTR_INTV times
	.equ	TSTATE		= 0x61	; for initializing USB
	.equ	NOW_CAPS	= 0x62	; for caps lock
	.equ	PREV_CAPS	= 0x63
	.equ	CAPS_STATE	= 0x64
	.equ	NOW_KANA	= 0x65	; for kana lock
	.equ	PREV_KANA	= 0x66
	.equ	KANA_STATE	= 0x67
	.equ	BUFF		= 0x68	; buffer for USB transfer
	.equ	KEYSTATUS	= 0x88	; buffer for 88's key status

	; a macro for reacting to the 88 key scan
	; this must be called very frequently while executing
	.macro SCAN_CHECK
	in 		XL,PIND	
	subi	XL,-LOW(KEYSTATUS)
	ld 		XL,X
	out		PORTB,XL
	.endmacro

	.cseg

	rjmp	VECT_RESET
	rjmp	VECT_INT0
	rjmp	VECT_INT1
	rjmp	VECT_TIMER1_CAPT
	rjmp	VECT_TIMER1_COMPA
	rjmp	VECT_TIMER1_OVF
	rjmp	VECT_TIMER0_OVF
	rjmp	VECT_USART0_RX
	rjmp	VECT_USART0_UDRE
	rjmp	VECT_USART0_TX
	rjmp	VECT_ANALOG_COMP
	rjmp	VECT_PCINT
	rjmp	VECT_TIMER1_COMPB
	rjmp	VECT_TIMER0_COMPA
	rjmp	VECT_TIMER0_COMPB
	rjmp	VECT_USI_START
	rjmp	VECT_USI_OVERFLOW
	rjmp	VECT_EE_READY
	rjmp	VECT_WDT_OVERFLOW

VECT_INT0:
VECT_INT1:
VECT_TIMER1_CAPT:
VECT_TIMER1_COMPA:
VECT_TIMER1_OVF:
VECT_TIMER0_OVF:
VECT_USART0_RX:
VECT_USART0_UDRE:
VECT_USART0_TX:
VECT_ANALOG_COMP:
VECT_PCINT:
VECT_TIMER1_COMPB:
VECT_TIMER0_COMPB:
VECT_USI_START:
VECT_USI_OVERFLOW:
VECT_EE_READY:
VECT_WDT_OVERFLOW:
	reti

	; ===== TIMER =====
	; called at every 1m sec
VECT_TIMER0_COMPA:
	mov		r14,XL
	SCAN_CHECK
	rcall	DO_USB
	SCAN_CHECK
	mov		XL,r14
	reti

	; ===== USB communication =====

	;
	; convert data on the ROM to the send format
	;
	; inputs:
	;   Z = source data pointer
	; outputs:
	;	*Z = converted data
	;
CONV_SEND:
	inc		ZL
	lpm
	mov		r18,r0
	inc		ZL
	subi	YL,-2
	push	YL
	;		r19				; temp
	;		r20				; source counter
	ldi		r21,1			; first data (phase) = 1
	ldi		r22,0			; six counter
	ldi		r23,0			; target counter
	;		r24				; target data
CNV_SND_LOOP1:
	ldi		r20,8
	lpm
	inc		ZL
CNV_SND_LOOP2:
	mov		r19,r0
	lsl		r0
	andi	r19,0x80
	brne	CNV_SND_NO_CHANGE_PHASE
CNV_SND_CHANGE_POL:			; msb = 0, change phase
	ldi		r22,0
	ldi		r19,1
	eor		r21,r19
	rjmp	CNV_SND_MERGE
CNV_SND_NO_CHANGE_PHASE:	; msb = 1, not change phase
	inc		r22
CNV_SND_MERGE:
	lsl		r24
	or		r24,r21
	inc		r23
	cpi		r23,8
	brne	CNV_SND_NO_STORE
	ldi		r23, 0
	st		Y+,r24
CNV_SND_NO_STORE:
	cpi		r22,6
	breq	CNV_SND_CHANGE_POL
CNV_SND_NO6:											
	dec		r20
	brne	CNV_SND_LOOP2
	dec		r18
	brne	CNV_SND_LOOP1
	push	r23
	cpi		r23,0
	breq	CNV_SND_END_END
CNV_SND_END_LOOP:
	lsl		r24
	inc		r23
	cpi		r23,8
	brne	CNV_SND_END_LOOP
	st		Y,r24
CNV_SND_END_END:
	pop		r20
	pop		r21
	mov		r19,YL
	sub		r19,r21
	mov		YL,r21
	subi	YL,2
	st		Y+,r19
	st		Y,r20
	ret

	;
	; send data on the RAM to the USB bus
	;
	; inputs:
	;   Y = send data pointer
	;
SEND:
	ld		r19,Y+
	ld		r20,Y+
	ldi		r22,32			; read counter
	lsl		r20
	lsl		r20
	ldi		r23, 0x02		; write data
	out		PORTD, r23
	ldi		r23, 0x3
	out		DDRD, r23
	lsl		r20				; 1
	lsl		r20				; 1
	lsl		r20				; 1
SEND_LOOP:
	nop						; 1
	nop						; 1
	subi	r22,32			; 1
	brne	SEND_WAIT		; 1/2
	ldi		r22,0			; 1
	ld		r21,Y+			; 2
	dec		r19				; 1
	brmi	SEND_END		; 1/2
	brne	SEND_NO_READ	; 1/2
	mov		r22,r20			; 1
SEND_NO_READ:
	ldi		r23,1			; 1
	sbrc	r21,7			; 1
	ldi		r23,2			; 1
	out		PORTD,r23		; 1
	lsl		r21				; 1
	rjmp	SEND_LOOP		; 2
SEND_WAIT:
	nop						; 1
	nop						; 1
	nop						; 1
	nop						; 1
	rjmp	SEND_NO_READ	; 2
SEND_EOP:
	ldi		r18, 0x02		; write data
	out		PORTD, r18
	ldi		r18, 0x3		; 1
	out		DDRD, r18		; 1
	nop						; 1
	ldi		r19,4			; 1
SEND_EOP_LOOP:
	dec		r19				; 1
	brne	SEND_EOP_LOOP	; 2/1
SEND_END:
	nop						; 1
	nop						; 1
	nop						; 1
	ldi		r19,0			; 1
	out		PORTD,r19		; 1
	ldi		r19,11			; 1
SEND_END_LOOP1:
	dec		r19				; 1
	brne	SEND_END_LOOP1	; 2/1
	nop						; 1
	ldi		r19,2			; 1
	out		PORTD,r19		; 1
	ldi		r19,11			; 1
SEND_END_LOOP2:
	dec		r19				; 1
	brne	SEND_END_LOOP2	; 2/1
	nop						; 1
	ldi		r19, 0x0		; 1
	out		DDRD, r19
	out		PORTD, r19
	ret

	;
	; receive data from USB
	;	input = Z : destination pointer
	;	output = *ZL : received data
	;
RECEIVE:
	push	ZL
	inc		ZL
	ldi		r19, 0
	ldi		r20, 0xff
	ldi		r21, 0xff
RECEIVE_WAIT:
	sbic	PIND, 1
	rjmp	RECEIVE_WAIT
RECEIVE_LOOP:
	in		r18, PIND			; 1
	andi	r18, 0x03 			; 1
	breq	RECEIVE_END_LOOP	; 1
	cp		r18,r20				; 1
	mov		r20,r18				; 1
	breq	RECEIVE_EQUAL		; 1/2
RECEIVE_NOT_EQUAL:
	andi	r21,0x3				; 1
	cp		r18,r21				; 1
	brne	RECEIVE_LONGER		; 1/2
RECEIVE_SHORTER:
	ror		r18					; 1
	ror		r22					; 1
	subi	r19,32				; 1
	in		r21,PIND			; 1
	brne	RECEIVE_NO_STORE	; 1/2
	st		Z+,r22				; 2
	rjmp	RECEIVE_LOOP		; 2
RECEIVE_NO_STORE_L:
	nop							; 1
RECEIVE_NO_STORE:
	rjmp	RECEIVE_LOOP		; 2
RECEIVE_EQUAL:
	nop							; 1
	nop							; 1
RECEIVE_LONGER:
	ror		r18					; 1
	ror		r22					; 1
	subi	r19,32				; 1
	in		r21,PIND			; 1
	brne	RECEIVE_NO_STORE_L	; 1/2
	st		Z+,r22				; 2
	rjmp	RECEIVE_LOOP		; 2
RECEIVE_END_LOOP:
	sbis	PIND, 1
	rjmp	RECEIVE_END_LOOP
	mov		r18,ZL
	subi	r18,LOW(BUFF)
	dec		r18
	pop		ZL
	st		Z,r18
	ret

	;
	; convert read data to actual data
	;   input = Y : source pointer
	;	        Z : destination pointer
	;   output = *Z : converted data
	;
CONV_RECEIVE:
	push	r18
	ld		r18,Y+
	ldi		r19, 1				; source counter
	ldi		r20, 8				; destination counter
	ldi		r23, 0xff			; previous value
	ldi		r24, 0				; NZRI count
	ldi		r25, 0				; result value
								; r21 : source value
								; r22 : temprorary
CONV_RCV_LOOP:
	dec		r19
	brne	CONV_RCV_NO_LOAD
	dec 	r18
	breq	CONV_RCV_END
	ldi		r19,8
	ld		r21, Y+
CONV_RCV_NO_LOAD:
	cpi		r24,6
	brne	CONV_RCV_UNDER6
	mov		r22,r21
	ror		r21
	andi	r22,1
	mov		r23,r22
	ldi		r24,0
	rjmp	CONV_RCV_LOOP
CONV_RCV_UNDER6:
	mov		r22,r21
	ror		r21
	andi	r22,1
	cp		r23,r22
	brne	CONV_RCV_NE
CONV_RCV_EQ:
	inc		r24
	sec
	rjmp	CONV_RCV_MERGE
CONV_RCV_NE:
	ldi		r24,0
	clc
CONV_RCV_MERGE:
	mov		r23,r22
	ror		r25
	dec		r20
	brne	CONV_RCV_LOOP
	ldi		r20,8
	st		Z+, r25
	rjmp	CONV_RCV_LOOP
CONV_RCV_END:
	pop		r18
	ret

	; ===== set address =====
SET_ADDRESS:
	; setup stage

	ldi		ZH, HIGH(PKT_SETUP_0*2)
	ldi		ZL, LOW(PKT_SETUP_0*2)
	ldi		YL,LOW(BUFF)
	rcall	CONV_SEND

	ldi		ZH, HIGH(PKT_SET_ADDRESS*2)
	ldi		ZL, LOW(PKT_SET_ADDRESS*2)
	ldi		YL,LOW(BUFF)+6
	rcall	CONV_SEND

	ldi		YL,LOW(BUFF)
	rcall	SEND

	ldi		YL,LOW(BUFF)+6	
	rcall	SEND

	ldi		ZL, LOW(BUFF)	;	should be ack
	rcall	RECEIVE

	; no data stage

	; status stage
SET_ADDRESS_NAK:
	ldi		ZH, HIGH(PKT_IN_0_0*2)
	ldi		ZL, LOW(PKT_IN_0_0*2)
	ldi		YL,LOW(BUFF)
	rcall	CONV_SEND

	ldi		ZH, HIGH(PKT_ACK*2)
	ldi		ZL, LOW(PKT_ACK*2)
	ldi		YL,LOW(BUFF+14)	
	rcall	CONV_SEND

	ldi		YL,LOW(BUFF)
	rcall	SEND

	ldi		ZL, LOW(BUFF)
	rcall	RECEIVE

	lds		r18,BUFF+2
	andi	r18,0x3
	breq	SET_ADDRESS_NAK
	
	; ack
	ldi		YL,LOW(BUFF+14)	
	rcall	SEND

	ldi		ZL,LOW(INIT_RETURN)
	ldi		ZH,HIGH(INIT_RETURN)
	ijmp

	; ===== set configuration =====
SET_CONFIGURATION:
	; setup stage
	ldi		ZH, HIGH(PKT_SETUP_1*2)
	ldi		ZL, LOW(PKT_SETUP_1*2)
	ldi		YL,LOW(BUFF)	
	rcall	CONV_SEND

	ldi		ZH, HIGH(PKT_SET_CONFIGURATION*2)
	ldi		ZL, LOW(PKT_SET_CONFIGURATION*2)
	ldi		YL,LOW(BUFF)+6
	rcall	CONV_SEND

	ldi		YL,LOW(BUFF)	
	rcall	SEND

	ldi		YL,LOW(BUFF)+6
	rcall	SEND

	ldi		ZL, LOW(BUFF)
	rcall	RECEIVE

	; no data stage

	; status stage
SET_CONF_NAK:
	ldi		ZH, HIGH(PKT_IN_1_0*2)
	ldi		ZL, LOW(PKT_IN_1_0*2)	
	ldi		YL,LOW(BUFF)	
	rcall	CONV_SEND

	ldi		ZH, HIGH(PKT_ACK*2)
	ldi		ZL, LOW(PKT_ACK*2)
	ldi		YL,LOW(BUFF+14)
	rcall	CONV_SEND

	ldi		YL,LOW(BUFF)	; pkt in
	rcall	SEND

	ldi		ZL, LOW(BUFF)
	rcall	RECEIVE

	lds		r18,BUFF+2
	andi	r18,0x3
	breq	SET_CONF_NAK

	ldi		YL,LOW(BUFF+14)	; ack
	rcall	SEND

	ldi		ZL,LOW(INIT_RETURN)
	ldi		ZH,HIGH(INIT_RETURN)
	ijmp

	; ===== interrupt transfer =====
INTR_TRANSFER:
	SCAN_CHECK
	ldi		ZH, HIGH(PKT_IN_1_1*2)
	ldi		ZL, LOW(PKT_IN_1_1*2)
	SCAN_CHECK
	ldi		YL,LOW(BUFF)
	SCAN_CHECK

	; <----- from here, can't check scan...
	ldi		r18,0xff
	out		PORTB,r18

	rcall	CONV_SEND
	ldi		ZH, HIGH(PKT_ACK*2)
	ldi		ZL, LOW(PKT_ACK*2)
	ldi		YL,LOW(BUFF+17)
	rcall	CONV_SEND
	ldi		YL,LOW(BUFF)
	rcall	SEND
	ldi		r18,0
	sts		(BUFF+5),r18
	ldi		ZL, LOW(BUFF+4)	; destination
	rcall	RECEIVE
	; ack
	ldi		YL,LOW(BUFF+17)	
	rcall	SEND

	; return to main if nak, stall nor not yet
	lds		r18,BUFF+6
	andi	r18,3
	brne	NO_EXIT
	ret
NO_EXIT:
	ldi		YL, LOW(BUFF+4)	; source
	ldi		ZL, LOW(BUFF+4)	; destination
	rcall	CONV_RECEIVE

	; init KEYSTATUS buffer
	rcall	INITKEY

	; detect caps and kana rising edge
	lds		r20,PREV_CAPS
	cpi		r20,0
	brne	NOT_CAPS_EDGE
	lds		r20,NOW_CAPS
	cpi		r20,1
	brne	NOT_CAPS_EDGE
	lds		r19, CAPS_STATE	; toggle caps state
	com		r19	
	sts		CAPS_STATE,r19
NOT_CAPS_EDGE:
	lds		r20,PREV_KANA
	cpi		r20,0
	brne	NOT_KANA_EDGE
	lds		r20,NOW_KANA
	cpi		r20,1
	brne	NOT_KANA_EDGE
	lds		r19, KANA_STATE	; toggle kana state
	com		r19	
	sts		KANA_STATE,r19	
NOT_KANA_EDGE:

	; make caps & kana lock
	lds		r19,CAPS_STATE
	ori		r19,0b01111111
	ldi		ZL,LOW(KEYSTATUS+0x0a*4)
	st		Z+,r19
	st		Z+,r19
	st		Z+,r19
	st		Z+,r19
	lds		r19,KANA_STATE
	ori		r19,0b11011111
	ldi		ZL,LOW(KEYSTATUS+0x08*4)
	st		Z+,r19
	st		Z+,r19
	st		Z+,r19
	st		Z+,r19
	
	; until here, can't check scan... ----->

	; delay caps and kana
	SCAN_CHECK
	lds		r20,NOW_CAPS
	SCAN_CHECK
	sts		PREV_CAPS,r20
	SCAN_CHECK
	lds		r20,NOW_KANA
	SCAN_CHECK
	sts		PREV_KANA,r20
	; clear caps and kana
	SCAN_CHECK
	ldi		r20,0
	SCAN_CHECK
	sts		NOW_CAPS,r20
	SCAN_CHECK
	sts		NOW_KANA,r20

	; BUFF+6 ... modifires BUFF+8 - +13 ... keys

	; make KEYSTATUS data

	; modifier keys

	SCAN_CHECK
	ldi		r20,0xff
	SCAN_CHECK
	lds		r18,BUFF+6
	SCAN_CHECK
	sbrc	r18,0
	andi	r20,0b01111111	
	SCAN_CHECK
	sbrc	r18,1
	andi	r20,0b10111111
	SCAN_CHECK
	sbrc	r18,2
	andi	r20,0b11101111
	SCAN_CHECK
	sbrc	r18,4
	andi	r20,0b01111111
	SCAN_CHECK
	sbrc	r18,5
	andi	r20,0b10111111
	SCAN_CHECK
	sbrc	r18,6
	andi	r20,0b11101111

	SCAN_CHECK
	ldi		ZL,LOW(KEYSTATUS+0x08*4)
	SCAN_CHECK

	ld		r19,Z
	and		r20,r19

	SCAN_CHECK
	st		Z+,r20
	SCAN_CHECK
	st		Z+,r20
	SCAN_CHECK
	st		Z+,r20
	SCAN_CHECK
	st		Z+,r20

	; normal keys
	SCAN_CHECK
	ldi		YL,LOW(BUFF+8)
	ldi		r19, SIMUL_PUSH
KEYLOOP6:
	SCAN_CHECK
	ld		r20,Y+
	; caps and kana check
	SCAN_CHECK
	cpi		r20,0x39
	brne	NOT_CAPS
	SCAN_CHECK
	ldi		r21,1
	SCAN_CHECK
	sts		NOW_CAPS,r21
NOT_CAPS:
	SCAN_CHECK
	cpi		r20,0x88
	brne	NOT_KANA
	SCAN_CHECK
	ldi		r21,1
	SCAN_CHECK
	sts		NOW_KANA,r21
NOT_KANA:
	SCAN_CHECK
	ldi		r21,0
	ldi		ZL,LOW(KEYMAP*2)
	SCAN_CHECK
	ldi		ZH,HIGH(KEYMAP*2)
	SCAN_CHECK
	add		ZL,r20
	adc		ZH,r21
	SCAN_CHECK
	lpm		r18,Z
	SCAN_CHECK
	cpi		r18,0xff	; if unassigned key...
	brne	NO_KEYIN2
	rjmp	NO_KEYIN
NO_KEYIN2:
	SCAN_CHECK
	mov		r21,r18		; r18 = r21 = pushed key
	andi	r18,0xf		; lower 4bit = address
	SCAN_CHECK
	lsl		r18
	lsl		r18
	SCAN_CHECK
	ldi		ZL,LOW(KEYSTATUS)
	add		ZL,r18		; Z = the value place
	SCAN_CHECK
	ld		r22,Z		; current value
	lsr		r21			; upper 4bit = bit
	SCAN_CHECK
	lsr		r21
	lsr		r21
	SCAN_CHECK
	lsr		r21
	ldi		r20,1
BITSHIFT:
	SCAN_CHECK
	dec		r21
	brmi	LOOPEXIT
	SCAN_CHECK
	lsl		r20
	SCAN_CHECK
	rjmp	BITSHIFT
LOOPEXIT:
	SCAN_CHECK
	com		r20		; r20 = and mask
	and		r22,r20	; mask the current value with r20
	SCAN_CHECK
	st		Z+,r22	; store value !
	SCAN_CHECK
	st		Z+,r22
	SCAN_CHECK
	st		Z+,r22
	SCAN_CHECK
	st		Z+,r22
NO_KEYIN:
	SCAN_CHECK
	dec		r19
	breq	KEYLOOPEND
	SCAN_CHECK
	ldi		ZL,LOW(KEYLOOP6)
	ldi		ZH,HIGH(KEYLOOP6)
	SCAN_CHECK
	ijmp
KEYLOOPEND:
	SCAN_CHECK
	ret

	; initialize KEYSTATUS buffer
INITKEY:
	ldi		ZL,LOW(KEYSTATUS)
	ldi		r19,16
	ldi		r18,0xff
INIKEYLP:
	st		Z+,r18
	st		Z+,r18
	st		Z+,r18
	st		Z+,r18
	dec		r19
	brne	INIKEYLP
	ret

	; clear BUFF for USB transfer
CLRBUFF:
	ldi		ZL,LOW(BUFF)
	ldi		r19,20
	ldi		r18,0x0
CBUFLP:
	st		Z+,r18
	dec		r19
	brne	CBUFLP
	ret

	; ===== MAIN ENTRANCE =====
VECT_RESET:
	ldi		r18,0xff
	out		DDRB,r18
	out		PORTB,r18
	ldi		r18, 0x0
	out		DDRD, r18

	ldi		r18, LOW(RAMEND)
	out		SPL, r18

	ldi		r18,0
	out		GIMSK,r18

	; wait for a while ...
	ldi		r18,255
DW1:
	ldi 	r19,255
DW2:
	ldi		r20,128
DW3:
	dec		r20
	brne	DW3
	dec		r19
	brne	DW2
	dec		r18
	brne	DW1

	; init variables
	ldi		r18,0
	sts		TCOUNT,r18
	sts		TSTATE,r18

	rcall	CLRBUFF

	; init key status buffer
	rcall	INITKEY

	; init caps and kana work area
	ldi		r18,0
	sts		NOW_CAPS,r18
	sts		PREV_CAPS,r18
	sts		NOW_KANA,r18
	sts		PREV_KANA,r18
	ldi		r18,0xff
	sts		CAPS_STATE,r18
	sts		KANA_STATE,r18

	; set timer as 1msec
	ldi		r18,TIMER_INTV
	out		OCR0A, r18
	ldi		r18,2
	out		TCCR0A, r18
	ldi		r18, 4			; prescale = 256 times
	out		TCCR0B, r18
	ldi		r18,1
	out		TIMSK, r18
	ldi		r18, 0;
	out		TCNT0, r18

	sei

	;===== MAIN LOOP =====
MAIN_LOOP: 	; main loop must be REALLY short!
	in 		XL,PIND				;1
	subi	XL,-LOW(KEYSTATUS)	;1
	ld 		XL,X				;2
	out		PORTB,XL			;1
	rjmp	MAIN_LOOP

	; USB main routine
	;   called from timer
DO_USB:
	; every 1ms here
	SCAN_CHECK
	lds		r18,TSTATE
	SCAN_CHECK
	cpi		r18,205
	brcs	NOT_SEND_EOP
	SCAN_CHECK
	rcall	SEND_EOP
	SCAN_CHECK
	lds		r18,TSTATE
NOT_SEND_EOP:
	SCAN_CHECK
	cpi		r18,217
	brne	INITIALIZE
	SCAN_CHECK
	lds		r18,TCOUNT
	SCAN_CHECK
	inc		r18
	SCAN_CHECK
	sts		TCOUNT,r18
	SCAN_CHECK
	cpi		r18,INTR_INTV
	breq	TIM_CONT
	SCAN_CHECK
	ret
TIM_CONT:
	; every INTR_INTV msec here
	SCAN_CHECK
	ldi		r18,0
	SCAN_CHECK
	sts		TCOUNT,r18
	SCAN_CHECK
	rjmp	INTR_TRANSFER	; receive DATA from USB
INITIALIZE:
	cli
	cpi		r18,200
	brne	INIT1
	ldi		r18, 0x3
	out		DDRD, r18
	ldi		r18, 0x00
	out		PORTD, r18
	rjmp	INIT_RETURN
INIT1:cpi	r18,204
	brne	INIT2
	ldi		r18, 0x0
	out		DDRD, r18
	ldi		r18, 0x00
	out		PORTD, r18
	rjmp	INIT_RETURN
INIT2:cpi	r18,214
	brne	INIT3
;	ldi		ZL,LOW(GET_REPORT)
;	ldi		ZH,HIGH(GET_REPORT)
;	ijmp
	rjmp	INIT_RETURN
INIT3:cpi		r18,215
	brne	INIT4
	ldi		ZL,LOW(SET_ADDRESS)
	ldi		ZH,HIGH(SET_ADDRESS)
	ijmp
INIT4:cpi		r18,216
	brne	INIT_RETURN
	ldi		ZL,LOW(SET_CONFIGURATION)
	ldi		ZH,HIGH(SET_CONFIGURATION)
	ijmp
INIT_RETURN:
	; state transition
	lds		r18,TSTATE
	inc		r18
	sts		TSTATE,r18
	sei
USB_EXIT:
	ret

	; ===== data on ROM =====
	; USB packets
PKT_SOF:
	.db		0,4
	.db		0b00000001,0b10100101	;	SYNC, SOF
	.db		0b00000000,0b00001000	;
PKT_ACK:
	.db		0,2
	.db		0b00000001,0b01001011	;	SYNC, ACK
PKT_SETUP_0:
	.db		0,4
	.db		0b00000001,0b10110100	;	SYNC, SETUP
	.db		0b00000000,0b00001000	;	ADDR = 0, ENDP = 0, CRC5
PKT_SET_ADDRESS:
	.db		0,12
	.db		0b00000001,0b11000011	;	SYNC, DATA0
	.db		0b00000000,0b10100000	;	OUT | Standard Request, SET_ADDRESS(5)
	.db		0b10000000,0b00000000	;	wValue = 1
	.db		0b00000000,0b00000000
	.db		0b00000000,0b00000000
	.db		0b11010111,0b10100100	;	CRC16	
PKT_SETUP_1:
	.db		0,4
	.db		0b00000001,0b10110100	;	SYNC, SETUP
	.db		0b10000000,0b00010111	;	ADDR = 1, ENDP = 0, CRC
PKT_SET_CONFIGURATION:
	.db		0,12
	.db		0b00000001,0b11000011	;	SYNC, DATA0
	.db		0b00000000,0b10010000	;	Out | Standard Request, SET_CONFIGURATION(9)
	.db		0b10000000,0b00000000	;	wValue = 1
	.db		0b00000000,0b00000000
	.db		0b00000000,0b00000000
	.db		0b11100100,0b10100100	;	CRC16
PKT_DUMMY_OUT:
	.db		0,4
	.db		0b00000001,0b11010010	;	SYNC, DATA1
	.db		0b00000000,0b00000000	;	CRC16
PKT_OUT_0_0:
	.db		0,4
	.db		0b00000001,0b10000111	;	SYNC, PID_OUT
	.db		0b00000000,0b00001000	;	ADDR = 0, ENDP = 0, CRC
PKT_IN_0_0:
	.db		0,4
	.db		0b00000001,0b10010110	;	SYNC, PID_IN
	.db		0b00000000,0b00001000	;	ADDR = 0, ENDP = 0, CRC
PKT_IN_0_1:
	.db		0,4
	.db		0b00000001,0b10010110	;	SYNC, PID_IN
	.db		0b00000000,0b00000101	;	ADDR = 0, ENDP = 1, CRC
PKT_IN_1_0:
	.db		0,4
	.db		0b00000001,0b10010110	;	SYNC, PID_IN
	.db		0b10000000,0b00010111	;	ADDR = 1, ENDP = 0, CRC
PKT_IN_1_1:
	.db		0,4
	.db		0b00000001,0b10010110	;	SYNC, PID_IN
	.db		0b10000001,0b00011010	;	ADDR = 1, ENDP = 1, CRC

	; a keymap from USB keycord to 88 keydata
KEYMAP:
	.db		0xff,0xff,0xff,0xff,0x12,0x22,0x32,0x42,	0x52,0x62,0x72,0x03,0x13,0x23,0x33,0x43
	.db		0x53,0x63,0x73,0x04,0x14,0x24,0x34,0x44,	0x54,0x64,0x74,0x05,0x15,0x25,0x16,0x26
	.db		0x36,0x46,0x56,0x66,0x76,0x07,0x17,0x06,	0x71,0x79,0x38,0x0a,0x69,0x75,0x65,0x02
	.db		0x35,0x45,0x55,0x37,0x27,0xff,0x47,0x57,	0x67,0x7a,0x19,0x29,0x39,0x49,0x59,0xff
	.db		0xff,0xff,0xff,0xff,0xff,0xff,0x4a,0x3a,	0x09,0xff,0x08,0x0b,0xff,0xff,0x1b,0x28
	.db		0x2a,0x1a,0x18,0xff,0x6a,0x21,0x5a,0x31,	0x71,0x10,0x20,0x30,0x40,0x50,0x60,0x70
	.db		0x01,0x11,0x00,0x61,0xff,0xff,0xff,0xff,	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff
	.db		0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff
	.db		0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x77,	0x58,0x45,0xff,0xff,0xff,0xff,0xff,0xff
	.db		0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff
	.db		0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff
	.db		0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff
	.db		0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff
	.db		0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff
	.db		0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff
	.db		0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff

